==> fuc.js-0.1.5.tgz: package/components/forms/select/select.js <==
import $ from 'jquery';
import Util from './util';

// Select 组件
const Select = (($) => {

  const NAME = 'select';
  const DATA_KEY = 'fuc.select';
  const EVENT_KEY = `.${DATA_KEY}`;
  const DATA_API_KEY = '.data-api';
  const JQUERY_NO_CONFLICT = $.fn[NAME];
  const ESCAPE_KEYCODE = 27; // KeyboardEvent.which value for Escape (Esc) key
  const SPACE_KEYCODE = 32; // KeyboardEvent.which value for space key
  const TAB_KEYCODE = 9; // KeyboardEvent.which value for tab key
  const ARROW_UP_KEYCODE = 38; // KeyboardEvent.which value for up arrow key
  const ARROW_DOWN_KEYCODE = 40; // KeyboardEvent.which value for down arrow key
  const RIGHT_MOUSE_BUTTON_WHICH = 3; // MouseEvent.which value for the right button (assuming a right-handed mouse)
  const REGEXP_KEYDOWN = new RegExp(`${ARROW_UP_KEYCODE}|${ARROW_DOWN_KEYCODE}|${ESCAPE_KEYCODE}`);

  // 事件
  const Event = {
    HIDE: `hide${EVENT_KEY}`,
    HIDDEN: `hidden${EVENT_KEY}`,
    SHOW: `show${EVENT_KEY}`,
    SHOWN: `shown${EVENT_KEY}`,
    SELECTED: `selected${EVENT_KEY}`,
    CLICK: `click${EVENT_KEY}`,
    CLICK_DATA_API: `click${EVENT_KEY}${DATA_API_KEY}`,
    KEYDOWN_DATA_API: `keydown${EVENT_KEY}${DATA_API_KEY}`,
    KEYUP_DATA_API: `keyup${EVENT_KEY}${DATA_API_KEY}`,
  };

  // 相关样式
  const ClassName = {
    OPTIONS: 'fuc-select-options',
    OPTION: 'fuc-select-option',
    TEXT: 'fuc-select-text',
    DISABLED: 'disabled',
    SELECT_ACTIVE: 'fuc-select-active',
    ACTIVE: 'active',
    EXPANDED: 'expanded',
    SHOW: 'show',
    DROPUP: 'dropup',
    DROPRIGHT: 'dropright',
    DROPLEFT: 'dropleft',
    MENURIGHT: 'dropdown-options-right',
    MENULEFT: 'dropdown-options-left',
    POSITION_STATIC: 'position-static',
  };

  const Selector = {
    SELECT: '.fuc-select',
    OPTIONS: '.fuc-select-options',
    OPTION: '.fuc-select-option',
    TEXT: '.fuc-select-text',
    INPUT: '.fuc-select-title > input',
    SELECT_ACTIVE: '.fuc-select-active',
    ACTIVE: '.active',
    DATA_TOGGLE: '[data-toggle="select"]',
  };

  const AttachmentMap = {
    TOP: 'top-start',
    TOPEND: 'top-end',
    BOTTOM: 'bottom-start',
    BOTTOMEND: 'bottom-end',
    RIGHT: 'right-start',
    RIGHTEND: 'right-end',
    LEFT: 'left-start',
    LEFTEND: 'left-end',
  };

  const Default = {
    offset: 0,
    flip: true,
    boundary: 'scrollParent',
  };

  const DefaultType = {
    offset: '(number|string|function)',
    flip: 'boolean',
    boundary: '(string|element)',
  };

  // 生成下拉选项的模板
  const Templates = {
    options: `<ul class="${ClassName.OPTIONS}">{{:options}}</ul>`,
    option: `<li class="${ClassName.OPTION}{{:selected}}" data-value="{{:value}}">
              <a class="${ClassName.TEXT}" href="#">{{:text}}</a>
            </li>`,
  };

  class Select {
    constructor(element, config) {
      this._element = element;
      this._config = this._getConfig(config);
      this._value = null; // 当前选中的值
      this._text = null; // 当前选中的文本
      // select options
      this._options = this._getOptionsElement();
      // input 输入框
      this._input = $(this._element).find(Selector.INPUT);

      // 初始化数据
      this.setOptionsData();

      this._addEventListeners();
    }

    // Getters
    static get Default() {
      return Default;
    }

    static get DefaultType() {
      return DefaultType;
    }

    // Public

    setOptionsData(data) {
      /**
       * optionsData 数据格式为
       [{
         value: 'value1',
         text: 'text1'
       }, {
         value: 'value2',
         text: 'text2',
         selected: true
       }, {
         value: 'value3',
         text: 'text3'
       }]
       */
      if (data) {
        this._config.optionsData = data;
      }
      const {optionsData} = this._config;
      if (optionsData && optionsData.length > 0) {
        this.render();
      } else {
        // 设置选中的值
        this.setSelectedOption();
      }
    }

    // 清空 select options
    clear() {
      $(this._options).empty();
      this._config.optionsData = null;
      this._value = null; // 当前选中的值
      this._text = null; // 当前选中的文本
      this._input.val('');
    }

    // 渲染组件
    render() {
      const {optionsData} = this._config;
      // TODO 这里先处理简单的一级选项
      const html = optionsData.map((item) => {
        const _item = {...item};
        if (_item.selected) {
          _item.selected = ` ${ClassName.ACTIVE}`;
          this.setSelectedOption({value: item.value, text: item.text});
        }
        return Util.applyTemplate(Templates.option, _item);
      });

      $(this._options).html(html);
    }

    // 设置 select 选项
    setSelectedOption({el, value, text} = {}) {
      // TODO 多选的情况待处理
      if (el) {
        $(this._options).find(Selector.ACTIVE).removeClass(ClassName.ACTIVE);
        $(el).addClass(ClassName.ACTIVE);
      }
      if (!value || !text) {
        const selectedOption = $(this._options).find(Selector.ACTIVE);
        value = value || selectedOption.data('value');
        text = text || selectedOption.find(Selector.TEXT).text();
      }
      this._value = value;
      if (value) {
        this._text = text;
        this._input.val(text);
      } else {
        this._text = '';
        this._input.val('');
      }
    }

    // 这里使用回调函数把数据返回
    getSelectedOption(callback) {
      if ($.isFunction(callback)) {
        callback({
          value: this._value || '',
          text: this._text || '',
          selectedOption: $(this._options).find(Selector.ACTIVE),
        });
      }
    }

    // 显示或隐藏下拉菜单
    toggle() {
      // 清空其他已打开的 select
      Select._clearOptions();

      if (this._element.disabled || $(this._element).hasClass(ClassName.DISABLED)) {
        return;
      }

      // 如果下拉选项为 0 时，则返回
      if ($(this._options).length === 0 || $(this._options).find(Selector.OPTION).length === 0) {
        return;
      }

      const $element = $(this._element);
      const isActive = $element.hasClass(ClassName.SELECT_ACTIVE);

      if (isActive) {
        return;
      }

      const relatedTarget = {
        relatedTarget: this._element,
      };
      const showEvent = $.Event(Event.SHOW, relatedTarget);

      $element.trigger(showEvent);

      if (showEvent.isDefaultPrevented()) {
        return;
      }

      this._element.focus();
      this._element.setAttribute('aria-expanded', true);

      // 显示下拉框
      $element.toggleClass(ClassName.SELECT_ACTIVE)
        .trigger($.Event(Event.SHOWN, relatedTarget));
    }

    // Private

    _addEventListeners() {
      // 展开事件
      $(this._element).on(Event.CLICK, (event) => {
        event.preventDefault();
        event.stopPropagation();
        this.toggle();
      });

      // 选中某一 Option
      $(this._element).on(Event.CLICK, Selector.OPTION, (event) => {
        event.preventDefault();
        event.stopPropagation();

        const {currentTarget} = event;
        const $el = $(currentTarget);
        const ariaLevel = $el.attr('aria-level');
        if (ariaLevel === 'branch') {
          $el.parent().find(`${Selector.OPTION} > ${Selector.OPTIONS}`)
            .removeClass(ClassName.EXPANDED);
          $el.find(`> ${Selector.OPTIONS}`).addClass(ClassName.EXPANDED);

          this._setActiveItem($el);
        } else {
          const value = $el.data('value');
          const text = $el.find(Selector.TEXT).text();
          this.setSelectedOption({el: $el, value, text});
          this._hideSelect(value, text);
        }
      });
    }

    // 设置当前选中的元素
    _setActiveItem($el) {
      if (this.previousActiveItems) {
        this.previousActiveItems.forEach((el) => {
          $(el).removeClass(ClassName.ACTIVE);
        });
      }

      $el.addClass(ClassName.ACTIVE);

      this.previousActiveItems = [];

      this.previousActiveItems.push($el);
      let parentNode = $el.parentsUntil(ClassName.OPTION);

      // 分支节点也设置 ACTIVE
      while (parentNode && parentNode.length > 0) {
        parentNode.addClass(ClassName.ACTIVE);
        this.previousActiveItems.unshift(parentNode);
        parentNode = $el.parentsUntil(ClassName.OPTION);
      }
    }

    // 返回 config
    _getConfig(config) {
      config = {
        ...this.constructor.Default,
        ...$(this._element).data(),
        ...config,
      };

      Util.typeCheckConfig(
        NAME,
        config,
        this.constructor.DefaultType,
      );

      return config;
    }

    _getOptionsElement() {
      if (!this._options) {
        const options = $(this._element).find(Selector.OPTIONS);
        if (options && options.length > 0) {
          this._options = options[0];
        } else {
          this._options = $(`<ul class="${ClassName.OPTIONS}"/>`).appendTo(this._element)[0];
        }
      }
      return this._options;
    }

    // 隐藏 select
    _hideSelect(value, text) {
      $(this._element).removeClass(ClassName.SELECT_ACTIVE)
        .trigger($.Event(Event.SELECTED, {
          relatedTarget: this._element,
          value,
          text: value ? text : '',
        }));
    }

    // Static

    static _jQueryInterface(config, params) {
      return this.each(function () {
        let data = $(this).data(DATA_KEY);
        const _config = typeof config === 'object' ? config : null;

        if (!data) {
          data = new Select(this, _config);
          $(this).data(DATA_KEY, data);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError(`No method named "${config}"`);
          }
          if (params) {
            data[config](params);
          } else {
            data[config]();
          }

        }
      });
    }

    // 关闭其他 select
    static _clearOptions(event) {
      // 过滤掉一些按键和鼠标右键
      if (event && (event.which === RIGHT_MOUSE_BUTTON_WHICH ||
          event.type === 'keyup' && event.which !== TAB_KEYCODE)) {
        return;
      }

      const selects = $.makeArray($(Selector.SELECT));
      for (let i = 0; i < selects.length; i++) {
        const $select = $(selects[i]);
        const context = $select.data(DATA_KEY);
        const relatedTarget = {
          relatedTarget: selects[i],
        };

        if (!context) {
          continue;
        }

        if (!$select.hasClass(ClassName.SELECT_ACTIVE)) {
          continue;
        }

        if (event && (event.type === 'click' &&
            (/input|textarea/i).test(event.target.tagName) || event.type === 'keyup' && event.which === TAB_KEYCODE) &&
          $.contains($select, event.target)) {
          continue;
        }

        const hideEvent = $.Event(Event.HIDE, relatedTarget);
        $select.trigger(hideEvent);
        if (hideEvent.isDefaultPrevented()) {
          continue;
        }

        selects[i].setAttribute('aria-expanded', 'false');

        $select
          .removeClass(ClassName.SELECT_ACTIVE)
          .trigger($.Event(Event.HIDDEN, relatedTarget));
      }
    }

    static _getParentFromElement(element) {
      let parent;
      const selector = Util.getSelectorFromElement(element);

      if (selector) {
        parent = $(selector)[0];
      }

      return parent || element.parentNode;
    }

    // eslint-disable-next-line complexity
    static _dataApiKeydownHandler(event) {
      if (/input|textarea/i.test(event.target.tagName)
          ? event.which === SPACE_KEYCODE || event.which !== ESCAPE_KEYCODE &&
          (event.which !== ARROW_DOWN_KEYCODE && event.which !== ARROW_UP_KEYCODE ||
            $(event.target).closest(Selector.MENU).length) : !REGEXP_KEYDOWN.test(event.which)) {
        return;
      }

      event.preventDefault();
      event.stopPropagation();

      if (this.disabled || $(this).hasClass(ClassName.DISABLED)) {
        return;
      }

      const parent = Select._getParentFromElement(this);
      const isActive = $(parent).hasClass(ClassName.SHOW);

      if (!isActive && (event.which !== ESCAPE_KEYCODE || event.which !== SPACE_KEYCODE) ||
        isActive && (event.which === ESCAPE_KEYCODE || event.which === SPACE_KEYCODE)) {
        if (event.which === ESCAPE_KEYCODE) {
          const toggle = $(parent).find(Selector.DATA_TOGGLE)[0];
          $(toggle).trigger('focus');
        }

        $(this).trigger('click');
        return;
      }

      const items = $(parent).find(Selector.VISIBLE_ITEMS).get();

      if (items.length === 0) {
        return;
      }

      let index = items.indexOf(event.target);

      if (event.which === ARROW_UP_KEYCODE && index > 0) { // Up
        index--;
      }

      if (event.which === ARROW_DOWN_KEYCODE && index < items.length - 1) { // Down
        index++;
      }

      if (index < 0) {
        index = 0;
      }

      items[index].focus();
    }
  }

  /**
   * ------------------------------------------------------------------------
   * Data Api implementation
   * ------------------------------------------------------------------------
   */

  $(document)
  //.on(Event.KEYDOWN_DATA_API, Selector.DATA_TOGGLE, Select._dataApiKeydownHandler)
  //.on(Event.KEYDOWN_DATA_API, Selector.MENU, Select._dataApiKeydownHandler)
    .on(`${Event.CLICK_DATA_API} ${Event.KEYUP_DATA_API}`, Select._clearOptions)
    .on(Event.CLICK_DATA_API, Selector.DATA_TOGGLE, function (event) {
      event.preventDefault();
      event.stopPropagation();
      Select._jQueryInterface.call($(this), 'toggle');
    });

  /**
   * ------------------------------------------------------------------------
   * jQuery
   * ------------------------------------------------------------------------
   */

  $.fn[NAME] = Select._jQueryInterface;
  $.fn[NAME].Constructor = Select;
  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Select._jQueryInterface;
  };

  return Select;
})($);

export default Select;

==> fuc.js-0.2.0.tgz: package/components/forms/select/select.js <==
import $ from 'jquery';
import Util from './util';

// Select 组件
const Select = (($) => {

  const NAME = 'select';
  const DATA_KEY = 'fuc.select';
  const EVENT_KEY = `.${DATA_KEY}`;
  const DATA_API_KEY = '.data-api';
  const JQUERY_NO_CONFLICT = $.fn[NAME];
  const ESCAPE_KEYCODE = 27; // KeyboardEvent.which value for Escape (Esc) key
  const SPACE_KEYCODE = 32; // KeyboardEvent.which value for space key
  const TAB_KEYCODE = 9; // KeyboardEvent.which value for tab key
  const ARROW_UP_KEYCODE = 38; // KeyboardEvent.which value for up arrow key
  const ARROW_DOWN_KEYCODE = 40; // KeyboardEvent.which value for down arrow key
  const RIGHT_MOUSE_BUTTON_WHICH = 3; // MouseEvent.which value for the right button (assuming a right-handed mouse)
  const REGEXP_KEYDOWN = new RegExp(`${ARROW_UP_KEYCODE}|${ARROW_DOWN_KEYCODE}|${ESCAPE_KEYCODE}`);

  // 事件
  const Event = {
    HIDE: `hide${EVENT_KEY}`,
    HIDDEN: `hidden${EVENT_KEY}`,
    SHOW: `show${EVENT_KEY}`,
    SHOWN: `shown${EVENT_KEY}`,
    SELECTED: `selected${EVENT_KEY}`,
    CLICK: `click${EVENT_KEY}`,
    CLICK_DATA_API: `click${EVENT_KEY}${DATA_API_KEY}`,
    KEYDOWN_DATA_API: `keydown${EVENT_KEY}${DATA_API_KEY}`,
    KEYUP_DATA_API: `keyup${EVENT_KEY}${DATA_API_KEY}`,
  };

  // 相关样式
  const ClassName = {
    OPTIONS: 'fuc-select-options',
    OPTION: 'fuc-select-option',
    TEXT: 'fuc-select-text',
    DISABLED: 'disabled',
    SELECT_ACTIVE: 'fuc-select-active',
    ACTIVE: 'active',
    EXPANDED: 'expanded',
    SHOW: 'show',
    DROPUP: 'dropup',
    DROPRIGHT: 'dropright',
    DROPLEFT: 'dropleft',
    MENURIGHT: 'dropdown-options-right',
    MENULEFT: 'dropdown-options-left',
    POSITION_STATIC: 'position-static',
  };

  const Selector = {
    SELECT: '.fuc-select',
    OPTIONS: '.fuc-select-options',
    OPTION: '.fuc-select-option',
    TEXT: '.fuc-select-text',
    INPUT: '.fuc-select-title > input',
    SELECT_ACTIVE: '.fuc-select-active',
    ACTIVE: '.active',
    DATA_TOGGLE: '[data-toggle="select"]',
  };

  const AttachmentMap = {
    TOP: 'top-start',
    TOPEND: 'top-end',
    BOTTOM: 'bottom-start',
    BOTTOMEND: 'bottom-end',
    RIGHT: 'right-start',
    RIGHTEND: 'right-end',
    LEFT: 'left-start',
    LEFTEND: 'left-end',
  };

  const Default = {
    offset: 0,
    flip: true,
    boundary: 'scrollParent',
  };

  const DefaultType = {
    offset: '(number|string|function)',
    flip: 'boolean',
    boundary: '(string|element)',
  };

  // 生成下拉选项的模板
  const Templates = {
    options: `<ul class="${ClassName.OPTIONS}">{{:options}}</ul>`,
    option: `<li class="${ClassName.OPTION}{{:selected}}" data-value="{{:value}}">
              <a class="${ClassName.TEXT}" href="#">{{:text}}</a>
            </li>`,
  };

  class Select {
    constructor(element, config) {
      this._element = element;
      this._config = this._getConfig(config);
      this._value = null; // 当前选中的值
      this._text = null; // 当前选中的文本
      // select options
      this._options = this._getOptionsElement();
      // input 输入框
      this._input = $(this._element).find(Selector.INPUT);

      // 初始化数据
      this.setOptionsData();

      this._addEventListeners();
    }

    // Getters
    static get Default() {
      return Default;
    }

    static get DefaultType() {
      return DefaultType;
    }

    // Public

    setOptionsData(data) {
      /**
       * optionsData 数据格式为
       [{
         value: 'value1',
         text: 'text1'
       }, {
         value: 'value2',
         text: 'text2',
         selected: true
       }, {
         value: 'value3',
         text: 'text3'
       }]
       */
      if (data) {
        this._config.optionsData = data;
      }
      const {optionsData} = this._config;
      if (optionsData && optionsData.length > 0) {
        this.render();
      } else {
        // 设置选中的值
        this.setSelectedOption();
      }
    }

    // 清空 select options
    clear() {
      $(this._options).empty();
      this._config.optionsData = null;
      this._value = null; // 当前选中的值
      this._text = null; // 当前选中的文本
      this._input.val('');
    }

    // 渲染组件
    render() {
      const {optionsData} = this._config;
      // TODO 这里先处理简单的一级选项
      const html = optionsData.map((item) => {
        const _item = {...item};
        if (_item.selected) {
          _item.selected = ` ${ClassName.ACTIVE}`;
          this.setSelectedOption({value: item.value, text: item.text});
        }
        return Util.applyTemplate(Templates.option, _item);
      });

      $(this._options).html(html);
    }

    // 设置 select 选项
    setSelectedOption({el, value, text} = {}) {
      // TODO 多选的情况待处理
      if (el) {
        $(this._options).find(Selector.ACTIVE).removeClass(ClassName.ACTIVE);
        $(el).addClass(ClassName.ACTIVE);
      }
      if (!value || !text) {
        const selectedOption = $(this._options).find(Selector.ACTIVE);
        value = value || selectedOption.data('value');
        text = text || selectedOption.find(Selector.TEXT).text();
      }
      this._value = value;
      if (value) {
        this._text = text;
        this._input.val(text);
      } else {
        this._text = '';
        this._input.val('');
      }
    }

    // 这里使用回调函数把数据返回
    getSelectedOption() {
      return {
        value: this._value || '',
        text: this._text || '',
        selectedOption: $(this._options).find(Selector.ACTIVE),
      };
    }

    // 显示或隐藏下拉菜单
    toggle() {
      // 清空其他已打开的 select
      Select._clearOptions();

      if (this._element.disabled || $(this._element).hasClass(ClassName.DISABLED)) {
        return;
      }

      // 如果下拉选项为 0 时，则返回
      if ($(this._options).length === 0 || $(this._options).find(Selector.OPTION).length === 0) {
        return;
      }

      const $element = $(this._element);
      const isActive = $element.hasClass(ClassName.SELECT_ACTIVE);

      if (isActive) {
        return;
      }

      const relatedTarget = {
        relatedTarget: this._element,
      };
      const showEvent = $.Event(Event.SHOW, relatedTarget);

      $element.trigger(showEvent);

      if (showEvent.isDefaultPrevented()) {
        return;
      }

      this._element.focus();
      this._element.setAttribute('aria-expanded', true);

      // 显示下拉框
      $element.toggleClass(ClassName.SELECT_ACTIVE)
        .trigger($.Event(Event.SHOWN, relatedTarget));
    }

    // Private

    _addEventListeners() {
      // 展开事件
      $(this._element).on(Event.CLICK, (event) => {
        event.preventDefault();
        event.stopPropagation();
        this.toggle();
      });

      // 选中某一 Option
      $(this._element).on(Event.CLICK, Selector.OPTION, (event) => {
        event.preventDefault();
        event.stopPropagation();

        const {currentTarget} = event;
        const $el = $(currentTarget);
        const ariaLevel = $el.attr('aria-level');
        if (ariaLevel === 'branch') {
          $el.parent().find(`${Selector.OPTION} > ${Selector.OPTIONS}`)
            .removeClass(ClassName.EXPANDED);
          $el.find(`> ${Selector.OPTIONS}`).addClass(ClassName.EXPANDED);

          this._setActiveItem($el);
        } else {
          const value = $el.data('value');
          const text = $el.find(Selector.TEXT).text();
          this.setSelectedOption({el: $el, value, text});
          this._hideSelect(value, text);
        }
      });
    }

    // 设置当前选中的元素
    _setActiveItem($el) {
      if (this.previousActiveItems) {
        this.previousActiveItems.forEach((el) => {
          $(el).removeClass(ClassName.ACTIVE);
        });
      }

      $el.addClass(ClassName.ACTIVE);

      this.previousActiveItems = [];

      this.previousActiveItems.push($el);
      let parentNode = $el.parentsUntil(ClassName.OPTION);

      // 分支节点也设置 ACTIVE
      while (parentNode && parentNode.length > 0) {
        parentNode.addClass(ClassName.ACTIVE);
        this.previousActiveItems.unshift(parentNode);
        parentNode = $el.parentsUntil(ClassName.OPTION);
      }
    }

    // 返回 config
    _getConfig(config) {
      config = {
        ...this.constructor.Default,
        ...$(this._element).data(),
        ...config,
      };

      Util.typeCheckConfig(
        NAME,
        config,
        this.constructor.DefaultType,
      );

      return config;
    }

    _getOptionsElement() {
      if (!this._options) {
        const options = $(this._element).find(Selector.OPTIONS);
        if (options && options.length > 0) {
          this._options = options[0];
        } else {
          this._options = $(`<ul class="${ClassName.OPTIONS}"/>`).appendTo(this._element)[0];
        }
      }
      return this._options;
    }

    // 隐藏 select
    _hideSelect(value, text) {
      $(this._element).removeClass(ClassName.SELECT_ACTIVE)
        .trigger($.Event(Event.SELECTED, {
          relatedTarget: this._element,
          value,
          text: value ? text : '',
        }));
    }

    // Static

    static _jQueryInterface(config, params) {
      // 处理单个 select 时，注意这里 this 是一个 jquery 对象
      if (this.length === 1 && config === 'getSelectedOption') {
        let data = this.data(DATA_KEY);

        if (!data) {
          data = new Select(this[0]);
          this.data(DATA_KEY, data);
        }

        return data[config]();
      }

      return this.each(function () {
        let data = $(this).data(DATA_KEY);
        const _config = typeof config === 'object' ? config : null;

        if (!data) {
          data = new Select(this, _config);
          $(this).data(DATA_KEY, data);
        }

        if (typeof config === 'string') {
          if (typeof data[config] === 'undefined') {
            throw new TypeError(`No method named "${config}"`);
          }
          if (params) {
            data[config](params);
          } else {
            data[config]();
          }

        }
      });
    }

    // 关闭其他 select
    static _clearOptions(event) {
      // 过滤掉一些按键和鼠标右键
      if (event && (event.which === RIGHT_MOUSE_BUTTON_WHICH ||
          event.type === 'keyup' && event.which !== TAB_KEYCODE)) {
        return;
      }

      const selects = $.makeArray($(Selector.SELECT));
      for (let i = 0; i < selects.length; i++) {
        const $select = $(selects[i]);
        const context = $select.data(DATA_KEY);
        const relatedTarget = {
          relatedTarget: selects[i],
        };

        if (!context) {
          continue;
        }

        if (!$select.hasClass(ClassName.SELECT_ACTIVE)) {
          continue;
        }

        if (event && (event.type === 'click' &&
            (/input|textarea/i).test(event.target.tagName) || event.type === 'keyup' && event.which === TAB_KEYCODE) &&
          $.contains($select, event.target)) {
          continue;
        }

        const hideEvent = $.Event(Event.HIDE, relatedTarget);
        $select.trigger(hideEvent);
        if (hideEvent.isDefaultPrevented()) {
          continue;
        }

        selects[i].setAttribute('aria-expanded', 'false');

        $select
          .removeClass(ClassName.SELECT_ACTIVE)
          .trigger($.Event(Event.HIDDEN, relatedTarget));
      }
    }

    static _getParentFromElement(element) {
      let parent;
      const selector = Util.getSelectorFromElement(element);

      if (selector) {
        parent = $(selector)[0];
      }

      return parent || element.parentNode;
    }

    // eslint-disable-next-line complexity
    static _dataApiKeydownHandler(event) {
      if (/input|textarea/i.test(event.target.tagName)
          ? event.which === SPACE_KEYCODE || event.which !== ESCAPE_KEYCODE &&
          (event.which !== ARROW_DOWN_KEYCODE && event.which !== ARROW_UP_KEYCODE ||
            $(event.target).closest(Selector.MENU).length) : !REGEXP_KEYDOWN.test(event.which)) {
        return;
      }

      event.preventDefault();
      event.stopPropagation();

      if (this.disabled || $(this).hasClass(ClassName.DISABLED)) {
        return;
      }

      const parent = Select._getParentFromElement(this);
      const isActive = $(parent).hasClass(ClassName.SHOW);

      if (!isActive && (event.which !== ESCAPE_KEYCODE || event.which !== SPACE_KEYCODE) ||
        isActive && (event.which === ESCAPE_KEYCODE || event.which === SPACE_KEYCODE)) {
        if (event.which === ESCAPE_KEYCODE) {
          const toggle = $(parent).find(Selector.DATA_TOGGLE)[0];
          $(toggle).trigger('focus');
        }

        $(this).trigger('click');
        return;
      }

      const items = $(parent).find(Selector.VISIBLE_ITEMS).get();

      if (items.length === 0) {
        return;
      }

      let index = items.indexOf(event.target);

      if (event.which === ARROW_UP_KEYCODE && index > 0) { // Up
        index--;
      }

      if (event.which === ARROW_DOWN_KEYCODE && index < items.length - 1) { // Down
        index++;
      }

      if (index < 0) {
        index = 0;
      }

      items[index].focus();
    }
  }

  /**
   * ------------------------------------------------------------------------
   * Data Api implementation
   * ------------------------------------------------------------------------
   */

  $(document)
  //.on(Event.KEYDOWN_DATA_API, Selector.DATA_TOGGLE, Select._dataApiKeydownHandler)
  //.on(Event.KEYDOWN_DATA_API, Selector.MENU, Select._dataApiKeydownHandler)
    .on(`${Event.CLICK_DATA_API} ${Event.KEYUP_DATA_API}`, Select._clearOptions)
    .on(Event.CLICK_DATA_API, Selector.DATA_TOGGLE, function (event) {
      event.preventDefault();
      event.stopPropagation();
      Select._jQueryInterface.call($(this), 'toggle');
    });

  /**
   * ------------------------------------------------------------------------
   * jQuery
   * ------------------------------------------------------------------------
   */

  $.fn[NAME] = Select._jQueryInterface;
  $.fn[NAME].Constructor = Select;
  $.fn[NAME].noConflict = function () {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Select._jQueryInterface;
  };

  return Select;
})($);

export default Select;
